package icontrols.marquee;

import com.ms.wd.app.Message;
import com.ms.wd.core.Closure;
import com.ms.wd.core.Component;
import com.ms.wd.ui.Color;
import com.ms.wd.ui.Control;
import com.ms.wd.ui.CreateEvent;
import com.ms.wd.ui.Font;
import com.ms.wd.ui.MouseEvent;
import com.ms.wd.ui.PaintEvent;
import com.ms.wd.ui.Rectangle;
import com.ms.wd.util.Root;
import com.ms.wd.win32.RECT;
import com.ms.wd.win32.Windows;
import icontrols.BorderDisplayer;

public class Marquee extends Control {
   public static final int BORDER_SINGLELINE = 1;
   public static final int BORDER_3D = 2;
   public static final int BORDER_NONE = 4;
   public static final int MARM_ELEMENTEXPIRED = 1024;
   public static final int MARM_ELEMENTREQUIRED = 1025;
   public static final int MARM_ELEMENTCLICK = 1026;
   public static final int MARM_ELEMENTWRAP = 1027;
   public static final int DIRECTION_HORIZONTAL = 0;
   public static final int DIRECTION_VERTICAL = 1;
   public static final int WRAP_FOREVER = -1;
   static final Object EVENT_ELEMENTEXPIRED = new Object();
   static final Object EVENT_ELEMENTREQUIRED = new Object();
   static final Object EVENT_ELEMENTCLICK = new Object();
   static final Object EVENT_ELEMENTWRAP = new Object();
   static final int NUM_STARTDIVISIONS = 1;
   static final int REFRESH_TIMER = 100;
   static final int MAX_DIVISIONS = 100;
   ElementContainer[] m_elementList = new ElementContainer[1];
   ElementContainer[] m_elementListEnd = new ElementContainer[1];
   int[] m_divisionOffset = new int[]{0, 200};
   int m_numDivisions = 1;
   BorderDisplayer m_borderDisplayer = new BorderDisplayer();
   int m_yScrollAmount = 0;
   int m_xScrollAmount = -4;
   boolean m_horizontalScrolling = true;
   int m_elementSpacing = 20;

   public int getDirection() {
      return this.m_horizontalScrolling ? 0 : 1;
   }

   void deleteEntities() {
      for(int i = 0; i < this.m_numDivisions; ++i) {
         this.m_elementList[i] = null;
         this.m_elementListEnd[i] = null;
      }

   }

   public void addOnElementClick(MarqueeElementEventHandler value) {
      ((Component)this).addEventHandler(EVENT_ELEMENTCLICK, value);
   }

   Rectangle getBorderBounds() {
      int iBorderWidth = this.m_borderDisplayer.getBorderWidth();
      Rectangle bounds = ((Control)this).getBounds();
      bounds.x = iBorderWidth;
      bounds.y = iBorderWidth;
      bounds.width -= 2 * iBorderWidth;
      bounds.height -= 2 * iBorderWidth;
      return bounds;
   }

   protected void onCreate(CreateEvent e) {
      super.onCreate(e);
      Rectangle rect = this.getBorderBounds();
      if (this.m_horizontalScrolling) {
         this.m_divisionOffset[0] = rect.y;
         this.m_divisionOffset[1] = rect.y + rect.height;
      } else {
         this.m_divisionOffset[0] = rect.x;
         this.m_divisionOffset[1] = rect.x + rect.width;
      }

   }

   public MarqueeElementEventHandler getOnElementExpired() {
      return (MarqueeElementEventHandler)((Component)this).getEventHandler(EVENT_ELEMENTEXPIRED);
   }

   protected void onElementExpired(MarqueeElementEvent e) {
      Closure handler = ((Component)this).getEventHandler(EVENT_ELEMENTEXPIRED);
      if (handler != null) {
         ((MarqueeElementEventHandler)handler).invoke(this, e);
      }

   }

   public boolean addText(String text) {
      IMarqueeElement marEle = new TextElement((Object)null, text, (Font)null, (Color)null, (Color)null);
      return this.addMarqueeElement(marEle);
   }

   public Divisions getDivisions() {
      Divisions divs = new Divisions();
      divs.number = this.m_numDivisions;
      divs.size = new int[divs.number];

      for(int i = 0; i < divs.number; ++i) {
         divs.size[i] = this.m_divisionOffset[i + 1] - this.m_divisionOffset[i];
      }

      return divs;
   }

   public void setDivisions(int divs) {
      if (divs >= 1 && divs <= 100) {
         Rectangle rect = this.getBorderBounds();
         this.deleteEntities();
         this.m_elementList = new ElementContainer[divs];
         this.m_elementList = new ElementContainer[divs];
         this.m_divisionOffset = new int[divs + 1];
         this.m_numDivisions = divs;
         int spacing;
         int currentOffset;
         if (this.m_horizontalScrolling) {
            spacing = rect.height / divs;
            currentOffset = rect.y;
            this.m_divisionOffset[divs] = rect.y + rect.height;
         } else {
            spacing = rect.width / divs;
            currentOffset = rect.x;
            this.m_divisionOffset[divs] = rect.x + rect.width;
         }

         for(int i = 0; i < divs; ++i) {
            this.m_divisionOffset[i] = currentOffset;
            currentOffset += spacing;
         }

      } else {
         throw new IllegalArgumentException("Illegal number of divisions");
      }
   }

   public void setDivisions(Divisions divs) {
      if (divs.number >= 1 && divs.number <= 100) {
         this.deleteEntities();
         this.m_elementList = new ElementContainer[divs.number];
         this.m_elementListEnd = new ElementContainer[divs.number];
         this.m_numDivisions = divs.number;
         this.m_divisionOffset = new int[this.m_numDivisions + 1];
         Rectangle rect = this.getBorderBounds();
         int currentOffset;
         if (this.m_horizontalScrolling) {
            currentOffset = rect.y;
         } else {
            currentOffset = rect.x;
         }

         for(int i = 0; i < this.m_numDivisions; ++i) {
            this.m_divisionOffset[i] = currentOffset;
            currentOffset += divs.size[i];
         }

         this.m_divisionOffset[this.m_numDivisions] = currentOffset;
      } else {
         throw new IllegalArgumentException("Illegal number of divisions");
      }
   }

   public void addOnElementRequired(MarqueeDivisionEventHandler value) {
      ((Component)this).addEventHandler(EVENT_ELEMENTREQUIRED, value);
   }

   public Divisions getRelativeDivisions() {
      return this.getDivisions();
   }

   public void setRelativeDivisions(Divisions divs) {
      if (divs.number >= 1 && divs.number <= 100) {
         int total = 0;
         Divisions newDivs = new Divisions();
         int realSize;
         if (this.m_horizontalScrolling) {
            realSize = this.getBorderBounds().height;
         } else {
            realSize = this.getBorderBounds().width;
         }

         newDivs.number = divs.number;
         newDivs.size = new int[divs.number];

         for(int i = 0; i < divs.number; ++i) {
            total += divs.size[i];
         }

         for(int i = 0; i < divs.number; ++i) {
            newDivs.size[i] = divs.size[i] * realSize / total;
         }

         this.setDivisions(newDivs);
      } else {
         throw new IllegalArgumentException("Illegal number of divisions");
      }
   }

   public int getElementSpacing() {
      return this.m_elementSpacing;
   }

   public void setElementSpacing(int spacing) {
      this.m_elementSpacing = spacing;
   }

   public MarqueeElementEventHandler getOnElementWrap() {
      return (MarqueeElementEventHandler)((Component)this).getEventHandler(EVENT_ELEMENTWRAP);
   }

   protected void onElementWrap(MarqueeElementEvent e) {
      Closure handler = ((Component)this).getEventHandler(EVENT_ELEMENTWRAP);
      if (handler != null) {
         ((MarqueeElementEventHandler)handler).invoke(this, e);
      }

   }

   public MarqueeDivisionEventHandler getOnElementRequired() {
      return (MarqueeDivisionEventHandler)((Component)this).getEventHandler(EVENT_ELEMENTREQUIRED);
   }

   protected void onElementRequired(MarqueeDivisionEvent e) {
      Closure handler = ((Component)this).getEventHandler(EVENT_ELEMENTREQUIRED);
      if (handler != null) {
         ((MarqueeDivisionEventHandler)handler).invoke(this, e);
      }

   }

   protected void wmElementExpired(Message m) {
      Object tag = null;
      if (m.wParam != 0) {
         tag = Root.get(m.wParam);
         Root.free(m.wParam);
      }

      this.onElementExpired(new MarqueeElementEvent(tag));
   }

   public Marquee() {
      ((Control)this).setStyle(2, true);
   }

   public int getTrackSpeed() {
      return this.m_horizontalScrolling ? -this.m_xScrollAmount : -this.m_yScrollAmount;
   }

   public void setTrackSpeed(int amount) {
      if (this.m_horizontalScrolling) {
         this.m_xScrollAmount = -amount;
      } else {
         this.m_yScrollAmount = -amount;
      }

   }

   protected void onMouseUp(MouseEvent e) {
      super.onMouseUp(e);

      for(int i = 0; i < this.m_numDivisions; ++i) {
         for(ElementContainer ec = this.m_elementList[i]; ec != null; ec = ec.next) {
            int iTag = 0;
            Object tag = ec.marEle.getTag(e.x, e.y);
            if (tag != null) {
               iTag = Root.alloc(tag);
               Windows.PostMessage(((Control)this).getHandle(), 1026, iTag, 0);
            }
         }
      }

   }

   protected void wndProc(Message m) {
      ElementContainer ec = null;
      ElementContainer ecLast = null;
      switch (m.msg) {
         case 1:
            Windows.SetTimer(((Control)this).getHandle(), 0, 100, 0);
            m.result = 0;
            break;
         case 275:
            Rectangle rectangle = this.getBorderBounds();
            RECT rect = rectangle.toRECT();
            Windows.ScrollWindow(((Control)this).getHandle(), this.m_xScrollAmount, this.m_yScrollAmount, rect, rect);

            for(int i = 0; i < this.m_numDivisions; ++i) {
               ecLast = null;

               for(ElementContainer var17 = this.m_elementList[i]; var17 != null; var17 = var17.next) {
                  var17.marEle.scroll(this.m_xScrollAmount, this.m_yScrollAmount);
                  Rectangle rec = var17.marEle.getBounds();
                  if (this.m_xScrollAmount < 0 && rec.x + rec.width < rectangle.x || this.m_xScrollAmount > 0 && rec.x > rectangle.x + rectangle.width || this.m_yScrollAmount < 0 && rec.y + rec.height < rectangle.y || this.m_yScrollAmount > 0 && rec.y > rectangle.y + rectangle.height) {
                     int iTag = 0;
                     Object tag = var17.marEle.getTag();
                     if (tag != null) {
                        iTag = Root.alloc(tag);
                     }

                     int wrapCount = var17.marEle.getWrapCount();
                     if (wrapCount > 0 | wrapCount == -1) {
                        Windows.PostMessage(((Control)this).getHandle(), 1027, iTag, 0);
                        ElementContainer newEc = new ElementContainer();
                        newEc.marEle = var17.marEle;
                        Rectangle entRect = newEc.marEle.getBounds();
                        Rectangle marRect = this.getBorderBounds();
                        int x;
                        int y;
                        if (this.m_xScrollAmount < 0 | this.m_yScrollAmount < 0) {
                           Rectangle endRect = this.m_elementListEnd[i].marEle.getBounds();
                           if (this.m_horizontalScrolling) {
                              x = Math.max(endRect.x + endRect.width + this.m_elementSpacing, marRect.x + marRect.width);
                              y = marRect.y;
                           } else {
                              x = marRect.x;
                              y = Math.max(endRect.y + endRect.height + this.m_elementSpacing, marRect.y + marRect.height);
                           }

                           this.m_elementListEnd[i].next = newEc;
                           this.m_elementListEnd[i] = newEc;
                        } else {
                           Rectangle begRect = this.m_elementList[i].marEle.getBounds();
                           if (this.m_horizontalScrolling) {
                              x = Math.min(marRect.x, begRect.x - this.m_elementSpacing) - entRect.width;
                              y = marRect.y;
                           } else {
                              x = marRect.x;
                              y = Math.min(marRect.y, begRect.y - this.m_elementSpacing) - entRect.height;
                           }

                           newEc.next = this.m_elementList[i];
                           this.m_elementList[i] = newEc;
                        }

                        newEc.marEle.setLocation(x, y);
                        if (wrapCount > 0) {
                           newEc.marEle.setWrapCount(wrapCount - 1);
                        }
                     } else {
                        Windows.PostMessage(((Control)this).getHandle(), 1024, iTag, 0);
                     }

                     if (var17 == this.m_elementListEnd[i]) {
                        this.m_elementListEnd[i] = ecLast;
                     }

                     if (ecLast == null) {
                        this.m_elementList[i] = var17.next;
                     } else {
                        ecLast.next = var17.next;
                     }
                  }

                  ecLast = var17;
               }

               if (this.m_elementList[i] != null) {
                  Rectangle recStart = this.m_elementList[i].marEle.getBounds();
                  Rectangle recStop = this.m_elementListEnd[i].marEle.getBounds();
                  if (this.m_xScrollAmount < 0 && recStop.x + recStop.width < rectangle.x + rectangle.width || this.m_xScrollAmount > 0 && recStart.x > rectangle.x || this.m_yScrollAmount < 0 && recStop.y + recStop.height < rectangle.y + rectangle.height || this.m_yScrollAmount > 0 && recStart.y > rectangle.y) {
                     Windows.PostMessage(((Control)this).getHandle(), 1025, i, 0);
                  }
               } else {
                  Windows.PostMessage(((Control)this).getHandle(), 1025, i, 0);
               }
            }
            break;
         case 1024:
            this.wmElementExpired(m);
            break;
         case 1025:
            this.wmElementRequired(m);
            break;
         case 1026:
            this.wmElementClick(m);
            break;
         case 1027:
            this.wmElementWrap(m);
         default:
            super.wndProc(m);
      }

   }

   protected void wmElementClick(Message m) {
      Object tag = null;
      if (m.wParam != 0) {
         tag = Root.get(m.wParam);
         Root.free(m.wParam);
      }

      this.onElementClick(new MarqueeElementEvent(tag));
   }

   public void addOnElementWrap(MarqueeElementEventHandler value) {
      ((Component)this).addEventHandler(EVENT_ELEMENTWRAP, value);
   }

   public int getBorderStyle() {
      int style = this.m_borderDisplayer.getBorderStyle();
      if ((style & 2) != 0) {
         return 2;
      } else if (style == 4) {
         return 4;
      } else if (style == 1) {
         return 1;
      } else {
         throw new RuntimeException("internal error: unknown border style");
      }
   }

   public void setBorderStyle(int style) {
      switch (style) {
         case 1:
            this.m_borderDisplayer.setBorderStyle(1);
            break;
         case 2:
            this.m_borderDisplayer.setBorderStyle(18);
            break;
         case 3:
         default:
            throw new IllegalArgumentException("No such border style");
         case 4:
            this.m_borderDisplayer.setBorderStyle(4);
      }

      ((Control)this).invalidate();
   }

   public MarqueeElementEventHandler getOnElementClick() {
      return (MarqueeElementEventHandler)((Component)this).getEventHandler(EVENT_ELEMENTCLICK);
   }

   protected void onElementClick(MarqueeElementEvent e) {
      Closure handler = ((Component)this).getEventHandler(EVENT_ELEMENTCLICK);
      if (handler != null) {
         ((MarqueeElementEventHandler)handler).invoke(this, e);
      }

   }

   protected void wmElementWrap(Message m) {
      Object tag = null;
      if (m.wParam != 0) {
         tag = Root.get(m.wParam);
         Root.free(m.wParam);
      }

      this.onElementWrap(new MarqueeElementEvent(tag));
   }

   public boolean addMarqueeElement(IMarqueeElement marEle) {
      return this.addMarqueeElement(marEle, 0, 1);
   }

   public boolean addMarqueeElement(IMarqueeElement marEle, int div, int span) {
      if (div < this.m_numDivisions && div >= 0 && span >= 1 && span + div <= this.m_numDivisions && marEle != null && (this.m_yScrollAmount != 0 || this.m_xScrollAmount != 0)) {
         int x = 0;
         int y = 0;
         Rectangle bounds = this.getBorderBounds();
         int maxX = bounds.x + bounds.width;
         int maxY = bounds.y + bounds.height;
         int minX = bounds.x;
         int minY = bounds.y;
         ElementContainer entCon = new ElementContainer();
         entCon.marEle = marEle;
         int interval = this.m_divisionOffset[div + span] - this.m_divisionOffset[div];
         int width;
         int height;
         if (this.m_horizontalScrolling) {
            width = 0;
            height = interval;
            y = this.m_divisionOffset[div];
         } else {
            height = 0;
            width = interval;
            x = this.m_divisionOffset[div];
         }

         marEle.create(this, width, height);
         Rectangle rec = marEle.getBounds();

         for(int i = div; i < div + span; ++i) {
            if (this.m_elementList[i] != null) {
               Rectangle loc = this.m_elementListEnd[i].marEle.getBounds();
               if (loc.x + loc.width > maxX) {
                  maxX = loc.x + loc.width;
               }

               if (loc.y + loc.height > maxY) {
                  maxY = loc.y + loc.height;
               }

               loc = this.m_elementList[i].marEle.getBounds();
               if (loc.x < minX) {
                  minX = loc.x;
               }

               if (loc.y < minY) {
                  minY = loc.y;
               }
            }
         }

         if (this.m_horizontalScrolling) {
            if (this.m_xScrollAmount < 0) {
               x = maxX + this.m_elementSpacing > bounds.x + bounds.width ? maxX + this.m_elementSpacing : bounds.x + bounds.width;
            } else {
               x = minX - this.m_elementSpacing < bounds.x ? minX - rec.width - this.m_elementSpacing : bounds.x - rec.width;
            }
         } else if (this.m_yScrollAmount < 0) {
            y = maxY + this.m_elementSpacing > bounds.y + bounds.height ? maxY + this.m_elementSpacing : bounds.y + bounds.height;
         } else {
            y = minY - this.m_elementSpacing < bounds.y ? minY - rec.height - this.m_elementSpacing : bounds.y - rec.height;
         }

         marEle.setLocation(x, y);
         if (this.m_xScrollAmount >= 0 && this.m_yScrollAmount >= 0) {
            for(int i = div; i < div + span; ++i) {
               if (this.m_elementList[i] == null) {
                  this.m_elementListEnd[i] = entCon;
               } else {
                  entCon.next = this.m_elementList[i];
               }

               this.m_elementList[i] = entCon;
               entCon = new ElementContainer();
               entCon.marEle = new EmptyElement();
               entCon.marEle.create(this, rec.width, rec.height);
               entCon.marEle.setLocation(x, y);
            }
         } else {
            for(int i = div; i < div + span; ++i) {
               if (this.m_elementList[i] == null) {
                  this.m_elementList[i] = entCon;
               } else {
                  this.m_elementListEnd[i].next = entCon;
               }

               this.m_elementListEnd[i] = entCon;
               entCon = new ElementContainer();
               entCon.marEle = new EmptyElement();
               entCon.marEle.create(this, rec.width, rec.height);
               entCon.marEle.setLocation(x, y);
            }
         }

         return true;
      } else {
         return false;
      }
   }

   public void addOnElementExpired(MarqueeElementEventHandler value) {
      ((Component)this).addEventHandler(EVENT_ELEMENTEXPIRED, value);
   }

   protected void onPaint(PaintEvent e) {
      int iBorderWidth = this.m_borderDisplayer.getBorderWidth();
      Rectangle bounds = ((Control)this).getBounds();
      RECT newRect = new RECT();
      bounds.x = 0;
      bounds.y = 0;
      this.m_borderDisplayer.paintIn(e.graphics, bounds, (Rectangle)null);
      bounds.x += iBorderWidth;
      bounds.y += iBorderWidth;
      bounds.width -= 2 * iBorderWidth;
      bounds.height -= 2 * iBorderWidth;
      Windows.IntersectRect(newRect, bounds.toRECT(), e.clipRect.toRECT());
      e.clipRect.x = newRect.left;
      e.clipRect.y = newRect.top;
      e.clipRect.width = newRect.right - newRect.left;
      e.clipRect.height = newRect.bottom - newRect.top;

      for(int i = 0; i < this.m_numDivisions; ++i) {
         for(ElementContainer ec = this.m_elementList[i]; ec != null; ec = ec.next) {
            ec.marEle.paintIn(e.clipRect, e.graphics);
         }
      }

   }

   protected void wmElementRequired(Message m) {
      this.onElementRequired(new MarqueeDivisionEvent(m.wParam));
   }

   public void setDirection(int dir) {
      boolean bHS = dir == 0;
      if (bHS != this.m_horizontalScrolling) {
         Divisions divs = this.getRelativeDivisions();
         this.m_horizontalScrolling = bHS;
         if (bHS) {
            this.m_xScrollAmount = this.m_yScrollAmount;
            this.m_yScrollAmount = 0;
         } else {
            this.m_yScrollAmount = this.m_xScrollAmount;
            this.m_xScrollAmount = 0;
         }

         this.setRelativeDivisions(divs);
      }

   }
}
